var _ = function(){
	var action = new PlugIn.Action(function(selection){
		var alert = new Alert("Outline to Slides", "This script will import the topics from the current OmniOutliner document to OmniGraffle as canvas slides.\n\nInclude each topic’s children as slide body content?")
		alert.addOption("Topic & Children")
		alert.addOption("Topic Only")
		alert.addOption("Cancel")
		alert.show(function(result){
			if (result == 2){
				throw new Error('script cancelled')
			} else {
				function outlineToObjectsArray(){
					var array = new Array()
					items = rootItem.descendents
					for(i = 0; i < items.length; i++){
						item = items[i]
						obj = new Object()
						obj.level = item.level
						obj.index = item.index
						obj.topic = item.topic
						array.push(obj)
					}
					return encodeURIComponent(JSON.stringify(array))
				}
				var scriptURL = URL.tellScript("omnioutliner", outlineToObjectsArray.toString() + '\n' + 'outlineToObjectsArray()')
				scriptURL.call(function(reply){
					reply = decodeURIComponent(reply)
					reply = JSON.parse(reply)
					// title and body size and position are based upon current canvas dimensions
					var cnvs = document.windows[0].selection.canvas
					var cnvsSize = cnvs.size
					var cW = cnvsSize.width
					var cH = cnvsSize.height
					var vM = Math.round(cH/12)
					var hM = Math.round(cW/12)
					var boxOffset = Math.round(cH/32)
					var titleSize = Math.round(cW/14)
					var bodySize = Math.round(cW/20)
					var lineHeightPercentage = 1.25
					var titleBoxRect = new Rect(hM, vM, cW - hM*2, titleSize * lineHeightPercentage)
					var bodyBoxRect = new Rect(hM, vM + titleBoxRect.height + boxOffset, cW - hM*2, cH - vM*2 - boxOffset - titleBoxRect.height)
					// font family defaults
					var titleFontPSName = 'HelveticaNeue-Medium'
					var bodyFontPSName = 'HelveticaNeue'
					for(i = 0; i < reply.length; i++){
						// retrieve item data
						obj = reply[i]
						objIndex = obj.index
						objLevel = obj.level
						objTopic = obj.topic
						// create new canvas for each top-level item
						if (objLevel == 1){
							cnvs = addCanvas()
							cnvs.size = cnvsSize
							cnvs.canvasSizingMode = CanvasSizingMode.Fixed
							// create a slide title box
							shape = cnvs.addShape('Rectangle',titleBoxRect)
							shape.name = 'Slide Title'
							shape.textHorizontalPadding = 0
							shape.textVerticalPadding = 0
							shape.text = objTopic
							shape.fontName = titleFontPSName
							shape.autosizing = TextAutosizing.Overflow
							if (result == 0){
								shape.textHorizontalAlignment = HorizontalTextAlignment.Left
							} else {
								shape.textHorizontalAlignment = HorizontalTextAlignment.Center
							}
							shape.textVerticalPlacement = VerticalTextPlacement.Middle
							shape.textSize = titleSize
							shape.fillColor = null
							shape.shadowColor = null
							shape.strokeColor = null
							if (result == 0){
								// create a slide body box
								shape = cnvs.addShape('Rectangle',bodyBoxRect)
								shape.name = 'Slide Body'
								shape.textHorizontalPadding = 0
								shape.textVerticalPadding = 0
								shape.text = ' ' // add space to enable setting font size
								shape.fontName = bodyFontPSName
								shape.autosizing = TextAutosizing.Overflow
								shape.textHorizontalAlignment = HorizontalTextAlignment.Left
								shape.textVerticalPlacement = VerticalTextPlacement.Top
								shape.textSize = bodySize
								shape.fillColor = null
								shape.shadowColor = null
								shape.strokeColor = null
							}
						} else if (result == 0 && objLevel == 2){
							bodyShape = cnvs.graphicWithName('Slide Body')
							bodyText = bodyShape.text
							if (bodyText == ''){
								bodyShape.text = '•' + ' ' + objTopic
							} else {
								bodyShape.text = bodyText + '\n' + '•' + ' ' + objTopic
							}
							bodyShape.textSize = bodySize
						}
					}
				})
			}
		})
	});

	// validation routine determines if the action menu item is enabled
	action.validate = function(selection){
		return true
	};

	return action;
}();
_;